# How to contribute ?
New ideas and contributions are more than welcome.

Read the [Code of conduct](../CODE_OF_CONDUCT.md) if you're unsure of
how you're doing things.

The issue tracker is on [gitlab](https://gitlab.com/experimentslabs/engine_elabs/issues). Feel free to open issues here.

## Translations
Translations are made with Gettext, and are located in `/locale`. If you
want to create a new translation, execute
`LANGUAGE=[XX] rake app:gettext:add_language` and edit the new files. You
can use tools like `poedit`, `virtaal` or `Qt 5 Linguist` to help you in the process.

If you create a new model, use `rake gettext:store_model_attributes` to
update the list of model attributes names.

Use `rake gettext:find` to update the `.po` files

Rake tasks:
```
rake app:gettext:add_language[language]     # add a new language
rake app:gettext:find                       # Update pot/po files
rake app:gettext:pack                       # Create mo-files
rake app:gettext:store_model_attributes     # write the model attributes to <locale_path>/model_attributes.rb
```

## Git workflow and code submission
  - Branches `master` and `develop` are _fast-forward_ only.
  - Make all the MR against `develop` or a release branch if it exists.
  - Every MR should pass the test suite.

## Testing
This project uses various tools to enforce code consistency and to run tests:

  - [Rubocop](http://batsov.com/rubocop/) to check the code style. Run
    `bundle exec rubocop` or `bundle exec rubocop -a` to fix some errors
    automatically.
  - [Brakeman](https://brakemanscanner.org/), to check the code against
    known vulnerabilities and possible security issues. You can launch
    it locally with `bundle exec brakeman`.
  - [RSpec](http://rspec.org/) for unit test. You can launch it locally
    with `bundle exec rspec`.
  - [Cucumber](https://cucumber.io/) for integration tests. Run
    `bundle exec cucumber` locally to test the code.
  - [Eslint](https://eslint.org/) for JS code style. Launch
    `yarn run lint:js` or `yarn run lint:js-fix` to check locally. It's
    not perfect, but it helps.
  - [StyleLint](http://stylelint.io/) for SCSS code style. Run
    `yarn run lint:sass` or `yarn run lint:sass-fix` locally.
  - [Haml-lint](https://github.com/brigade/haml-lint) for views
    consistency. Run `bundle exec haml-lint`.
